@@ -8,6 +8,7 @@ from __future__ import absolute_import
 
 # Import Salt Testing Libs
 from salttesting import skipIf, TestCase
+from salt.exceptions import SaltInvocationError
 from salttesting.helpers import ensure_in_syspath
 from salttesting.mock import (
     MagicMock,
@@ -200,6 +201,64 @@ class DockerngTestCase(TestCase):
             client_timeout=60)
         dockerng_start.assert_called_with('cont')
 
+    def test_running_with_udp_bindings(self):
+        '''
+        Check that `ports` contains ports defined from `port_bindings` with
+        protocol declaration passed as tuple. As stated by docker-py
+        documentation
+
+        https://docker-py.readthedocs.io/en/latest/port-bindings/
+
+        In sls:
+
+        .. code-block:: yaml
+
+            container:
+                dockerng.running:
+                    - port_bindings:
+                        - '9090:9797/udp'
+
+        is equivalent of:
+
+        .. code-block:: yaml
+
+            container:
+                dockerng.running:
+                    - ports:
+                        - 9797/udp
+                    - port_bindings:
+                        - '9090:9797/udp'
+        '''
+        dockerng_create = Mock()
+        dockerng_start = Mock()
+        dockerng_inspect_image = Mock(return_value={
+            'Id': 'abcd',
+            'Config': {'ExposedPorts': {}}
+        })
+        __salt__ = {'dockerng.list_containers': MagicMock(),
+                    'dockerng.list_tags': MagicMock(),
+                    'dockerng.pull': MagicMock(),
+                    'dockerng.state': MagicMock(),
+                    'dockerng.inspect_image': dockerng_inspect_image,
+                    'dockerng.create': dockerng_create,
+                    'dockerng.start': dockerng_start,
+                    }
+        with patch.dict(dockerng_state.__dict__,
+                        {'__salt__': __salt__}):
+            dockerng_state.running(
+                'cont',
+                image='image:latest',
+                port_bindings=['9090:9797/udp'])
+        dockerng_create.assert_called_with(
+            'image:latest',
+            validate_input=False,
+            name='cont',
+            ports=[(9797, 'udp')],
+            port_bindings={'9797/udp': [9090]},
+            validate_ip_addrs=False,
+            client_timeout=60)
+        dockerng_start.assert_called_with('cont')
+
     def test_running_compare_images_by_id(self):
         '''
         Make sure the container is running
@@ -510,7 +569,7 @@ class DockerngTestCase(TestCase):
                                'result': True,
                                })
 
-    def test_running_discard_wrong_environemnt_values(self):
+    def test_running_discard_wrong_environment_values(self):
         '''
         environment values should be string.
         It is easy to write wrong sls this way
@@ -550,7 +609,7 @@ class DockerngTestCase(TestCase):
                 self.assertEqual(ret,
                                  {'changes': {},
                                   'comment': 'Environment values must'
-                                  ' be strings KEY={0!r}'.format(wrong_value),
+                                  ' be strings KEY=\'{0}\''.format(wrong_value),
                                   'name': 'cont',
                                   'result': False})
 
@@ -581,6 +640,50 @@ class DockerngTestCase(TestCase):
             labels=['LABEL1', 'LABEL2'],
             client_timeout=60)
 
+    def test_running_with_labels_from_image(self):
+        '''
+        Test dockerng.running with labels parameter supports also
+        labels carried by the image.
+        '''
+        dockerng_create = Mock()
+
+        image_id = 'a' * 128
+        dockerng_inspect_image = MagicMock(
+            return_value={
+                'Id': image_id,
+                'Config': {
+                    'Hostname': 'saltstack-container',
+                    'WorkingDir': '/',
+                    'Cmd': ['bash'],
+                    'Volumes': {'/path': {}},
+                    'Entrypoint': None,
+                    'ExposedPorts': {},
+                    'Labels': {'IMAGE_LABEL': 'image_foo',
+                               'LABEL1': 'label1'},
+                },
+                })
+        __salt__ = {'dockerng.list_containers': MagicMock(),
+                    'dockerng.list_tags': MagicMock(),
+                    'dockerng.pull': MagicMock(),
+                    'dockerng.state': MagicMock(),
+                    'dockerng.inspect_image': dockerng_inspect_image,
+                    'dockerng.create': dockerng_create,
+                    }
+        with patch.dict(dockerng_state.__dict__,
+                        {'__salt__': __salt__}):
+            dockerng_state.running(
+                'cont',
+                image='image:latest',
+                labels=[{'LABEL1': 'foo1'}, {'LABEL2': 'foo2'}],
+                )
+        dockerng_create.assert_called_with(
+            'image:latest',
+            validate_input=False,
+            validate_ip_addrs=False,
+            name='cont',
+            labels={'LABEL1': 'foo1', 'LABEL2': 'foo2'},
+            client_timeout=60)
+
     def test_network_present(self):
         '''
         Test dockerng.network_present
@@ -769,6 +872,31 @@ class DockerngTestCase(TestCase):
                                            'removed': 'removed'},
                                'result': True})
 
+    def test_volume_present_wo_existing_volumes(self):
+        '''
+        Test dockerng.volume_present without existing volumes.
+        '''
+        dockerng_create_volume = Mock(return_value='created')
+        dockerng_remove_volume = Mock(return_value='removed')
+        __salt__ = {'dockerng.create_volume': dockerng_create_volume,
+                    'dockerng.remove_volume': dockerng_remove_volume,
+                    'dockerng.volumes': Mock(return_value={'Volumes': None}),
+                    }
+        with patch.dict(dockerng_state.__dict__,
+                        {'__salt__': __salt__}):
+            ret = dockerng_state.volume_present(
+                'volume_foo',
+                driver='bar',
+                force=True,
+                )
+        dockerng_create_volume.assert_called_with('volume_foo',
+                                                  driver='bar',
+                                                  driver_opts=None)
+        self.assertEqual(ret, {'name': 'volume_foo',
+                               'comment': '',
+                               'changes': {'created': 'created'},
+                               'result': True})
+
     def test_volume_absent(self):
         '''
         Test dockerng.volume_absent
@@ -956,6 +1084,121 @@ class DockerngTestCase(TestCase):
                                            name='cont',
                                            client_timeout=60)
 
+    def test_validate_input_min_docker_py(self):
+        docker_mock = Mock()
+        docker_mock.version_info = (1, 0, 0)
+        dockerng_mod.docker = None
+        with patch.dict(dockerng_mod.VALID_CREATE_OPTS['command'],
+                        {'path': 'Config:Cmd',
+                         'image_path': 'Config:Cmd',
+                         'min_docker_py': (999, 0, 0)}):
+            with patch.object(dockerng_mod, 'docker', docker_mock):
+                self.assertRaisesRegexp(SaltInvocationError,
+                                        "The 'command' parameter requires at"
+                                        " least docker-py 999.0.0.*$",
+                                        dockerng_state._validate_input,
+                                        {'command': 'echo boom'})
+
+    def test_command_defined_on_image_layer_dont_diff_if_attempted_to_blank(self):
+        '''
+        Assuming the docker image has a command defined, like ``sh``.
+        Erasing this value on sls level will not delete the command
+        in the container. And such the diff shouldn't be reported.
+        Assuming also the container is already running.
+
+        1. define your sls
+
+        .. code-block:: yaml
+
+            cont:
+                dockerng.running:
+                    - image: image:latest
+
+
+        2. run state.highstate
+
+        No diff should be reported
+        '''
+        image_id = 'abcdefg'
+        dockerng_create = Mock(return_value=True)
+        dockerng_start = Mock()
+        dockerng_list_containers = Mock(return_value=['cont'])
+        dockerng_inspect_container = Mock(
+            side_effect=[{
+                'Config': {
+                    'Image': 'image:latest',
+                    'Tty': False,
+                    'Labels': {},
+                    'Domainname': '',
+                    'User': '',
+                    'AttachStderr': True,
+                    'AttachStdout': True,
+                    'Hostname': 'saltstack-container',
+                    'Env': [],
+                    'WorkingDir': '/',
+                    'Cmd': None,
+                    'Volumes': {},
+                    'Entrypoint': None,
+                    'ExposedPorts': {},
+                    'OpenStdin': False,
+                },
+                'HostConfig': {
+                    'PublishAllPorts': False,
+                    'Dns': [],
+                    'Links': None,
+                    'CpusetCpus': '',
+                    'RestartPolicy': {'MaximumRetryCount': 0, 'Name': ''},
+                    'CapAdd': None,
+                    'NetworkMode': 'default',
+                    'PidMode': '',
+                    'MemorySwap': 0,
+                    'ExtraHosts': None,
+                    'PortBindings': None,
+                    'LxcConf': None,
+                    'DnsSearch': [],
+                    'Privileged': False,
+                    'Binds': [],
+                    'Memory': 0,
+                    'VolumesFrom': None,
+                    'CpuShares': 0,
+                    'CapDrop': None,
+                },
+                'NetworkSettings': {
+                    'MacAddress': '00:00:00:00:00:01',
+                },
+                'Image': image_id}]
+        )
+        dockerng_inspect_image = MagicMock(
+            return_value={
+                'Id': image_id,
+                'Config': {
+                    'Hostname': 'saltstack-container',
+                    'WorkingDir': '/',
+                    'Cmd': ['bash'],  # !!! Cmd defined on Image
+                    'Volumes': {},
+                    'Entrypoint': None,
+                    'ExposedPorts': {},
+                },
+                })
+        __salt__ = {'dockerng.list_containers': dockerng_list_containers,
+                    'dockerng.inspect_container': dockerng_inspect_container,
+                    'dockerng.inspect_image': dockerng_inspect_image,
+                    'dockerng.list_tags': MagicMock(side_effect=[['image:latest']]),
+                    'dockerng.state': MagicMock(side_effect=['running']),
+                    }
+        with patch.dict(dockerng_state.__dict__,
+                        {'__salt__': __salt__}):
+            ret = dockerng_state.running(
+                'cont',
+                image='image:latest',
+                )
+        self.assertEqual(ret, {'name': 'cont',
+                               'comment': "Container 'cont' is already"
+                               " configured as specified",
+                               'changes': {},
+                               'result': True,
+                               })
+
 
 if __name__ == '__main__':
     from integration import run_tests
